"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function optimisticUi(params) {
    const { commitTransaction, onChangePendingTransactionCount, onError, onRerenderKind, optimisticUIEnabled } = params;
    let pendingTransactions = [];
    return {
        addTransaction: function (transaction) {
            const { kind, id } = transaction;
            pendingTransactions.push(transaction);
            onChangePendingTransactionCount();
            // Rerender immediately to show optimistic update
            onRerenderKind(kind, id);
            commitTransaction(transaction)
                .then(function () {
                pendingTransactions = pendingTransactions.filter(t => t !== transaction);
                onChangePendingTransactionCount();
                // We have removed the pending transaction asynchronously,
                // and the server may have already added changes on top,
                // which the UI does not show yet.
                onRerenderKind(kind, id);
            })
                .catch(onError);
        },
        pendingTransactionCount: () => pendingTransactions.length,
        withPendingTransactions: function (data) {
            if (!optimisticUIEnabled())
                return data;
            const result = {};
            Object.keys(data).forEach(k => {
                result[k] = Object.assign({}, data[k]);
            });
            // Apply pending transactions on top of the canon version
            pendingTransactions.forEach(function (transaction) {
                if ("delta" in transaction) {
                    const prev = result[transaction.kind][transaction.id];
                    // If we have not fetched the item yet,
                    // we don’t need to transform it optimistically
                    if (!prev)
                        return;
                    result[transaction.kind][transaction.id] = {
                        revision: "optimistic",
                        value: transaction.delta(prev.value)
                    };
                }
                else {
                    result[transaction.kind][transaction.id] = {
                        revision: "optimistic",
                        value: transaction.creation
                    };
                }
            });
            return result;
        }
    };
}
exports.optimisticUi = optimisticUi;
//# sourceMappingURL=optimisticUi.js.map